ggQQ <- function(LM) # argument: a linear model
{
    y <- quantile(LM$resid[!is.na(LM$resid)], c(0.25, 0.75))
    x <- qnorm(c(0.25, 0.75))
    slope <- diff(y)/diff(x)
    int <- y[1L] - slope * x[1L]
    p <- ggplot(LM, aes(sample=.resid)) +
        stat_qq(alpha = 0.5) +
        geom_abline(slope = slope, intercept = int, color="blue")

    return(p)
}

shinyServer(function(input, output,session) {
           
    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
        else return(paste(red,"<h4>Warning: ggplot2 graphics package not found, go back to base graphics</h4>"))
        if(gotLibs()[2]) library(grid)
        else return(paste(red,"<h4>Warning: grid graphics package not found, go back to base graphics</h4>"))        
        NULL       
    })
               
   data <- reactive({
       if(input$dataset=="Cancer Survival") 
            x <- read.table("cancer.R",header=TRUE)
       if(input$dataset=="Wells") 
            x <- read.table("wells.R",header=TRUE)[,c(2,1)]   
      if(input$dataset=="Mammals") 
          x <- read.table("mammals.R",header=TRUE)
      if(input$dataset=="Forbes 500") {
          x <- read.table("forbes.R",header=TRUE)
          if(input$var1=="Sector")
              x <- x[,c(input$var2,input$var1)]
          else                  
              x <- x[,c(input$var1,input$var2)]
      }   
      if(input$dataset=="Billionaires 1992") {
          x <- read.table("billionaires.R",header=TRUE)
          if(input$var3=="Region")
              x <- x[,c(input$var4,input$var3)]
          else                  
              x <- x[,c(input$var3,input$var4)]
      }         
      if(input$dataset=="Upload your own") {
          inFile <- input$file1
          if (is.null(inFile))
                return(NULL)
          x<-read.csv(inFile$datapath, header = input$header,
             sep = input$sep, quote = input$quote)
          if(dim(x)[[2]]==1) x <- cbind(x[[1]],1)
      }    
      if(length(table(x[,1]))<10)   x <- x[,2:1]
      x              
   
   })         

   transx <- function(x) {
      if(input$transx=="None") return(x)
      if(input$transx=="Square Root") return(sqrt(x))
      if(input$transx=="Log") return(log(x))
      if(input$transx=="Inverse") return(1/x)   
      if(input$transx=="Enter you own!") {
            f<-function(x) {
                eval(parse(text=input$transxx))
            }
            return(f(x))
      }      
   }

   transy <- function(x) {
      if(input$transy=="Same as X") return(transx(x))
      if(input$transy=="None") return(x)
      if(input$transy=="Square Root") return(sqrt(x))
      if(input$transy=="Log") return(log(x))
      if(input$transy=="Inverse") return(1/x)   
      if(input$transy=="Enter you own!") {
            f<-function(x) {
                eval(parse(text=input$transxy))
            }
            return(f(x))
      } 
   }    

   output$story <- renderText({
      txt <- ""
      if(input$dataset=="Cancer Survival")
          txt <- "<h4>Patients with advanced cancers of the stomach, bronchus, colon, ovary or breast were 
          treated with ascorbate (Vitamin C). The purpose of the study was to determine if patient survival 
          differed with respect to the organ affected by the cancer.<br>  
          Cameron, E. and Pauling, L. (1978) Supplemental ascorbate in the supportive treatment of cancer: 
          re-evaluation of prolongation of survival times in terminal human cancer. 
          Proceedings of the National Academy of Science USA, 75, 4538Ð4542.</h4>"

      if(input$dataset=="Wells")
          txt <- "<h4>The specific capacity of wells in the Appalachian mountain region of Pennsylvania has been measured 
          in four rock types. (Knopman 1990) The rock types are dolomite, limestone, siliclastic and metamorphic. 
          The capacities are recorded in gal/min/ft. </h4>"
          
      if(input$dataset=="Mammals")
          txt <- "<h4>Brain (in grams) and Body Weight (in kg) of 62 Mammals.</h4>"
          
      if(input$dataset=="Forbes 500")
          txt <- "<h4>This dataset holds several facts about 77 companies selected from the Forbes 500 list for 1986. 
          <br<This is a 1/10 systematic sample from the alphabetical list of companies. 
          <br>The Forbes 500 includes all companies in the top 500 on any of the criteria, and thus has almost 
          800 companies in the list.</h4>"
      if(input$dataset=="Billionaires 1992")
          txt <- "<h4> Fortune magazine publishes the list of billionaires annually. The 1992 list included 233 
          individuals or families. Their wealth, age and geographic location (Asia, Europe, Middle East, United States 
          or Other) is reported.</h4>"    
      txt        
   })
   
   output$plot1 <- renderPlot({
        
        x<-data()
        doBoxplot <- FALSE
        if(length(table(x[,2]))<10) doBoxplot <- TRUE
        
        if(input$transx=="None") xname <- colnames(x)[1]
        if(input$transx=="Square Root") xname <- paste("sqrt(",colnames(x)[1],")",sep="")
        if(input$transx=="Log") xname <- paste("log(",colnames(x)[1],")",sep="")
        if(input$transx=="Inverse") xname <- paste("1/",colnames(x)[1],sep="")
        if(input$transx=="Enter you own!") xname <- "T(x)"
        
        if(input$transy=="Same as X") whichtrans <- input$transx
        else whichtrans <- input$transy
        if(whichtrans=="None") yname <- colnames(x)[2]
        if(whichtrans=="Square Root") yname <- paste("sqrt(",colnames(x)[2],")",sep="")
        if(whichtrans=="Log") yname <- paste("log(",colnames(x)[2],")",sep="")
        if(whichtrans=="Inverse") yname <- paste("1/",colnames(x)[2],sep="")
        if(input$transx=="Enter you own!") yname <- "T(y)"

        if(input$whichgraph=="base") {
            if(doBoxplot) {   
                par(mfrow=c(1,2))               
                y <- transx(x[,1])
                fit <- aov(y~x[,2])
                boxplot(y~x[,2],main=list(xname,cex=1.5))
                qqnorm(residuals(fit))
                qqline(residuals(fit))
            }      
            else {
                par(mfrow=c(2,2))
                Dt <- data()  
                x <- transx(Dt[,1])
                y <- transy(Dt[,2])
                boxplot(x,main=list(xname,cex=1.5),horizontal=T)
                plot(c(0, 1), c(0, 1), axes = F, xlab = "", ylab = "", type = "n")
                plot(x,y,xlab=list(xname,cex=1.5),ylab=list(yname,cex=1.5))
                boxplot(y,main=list(yname,cex=1.5))         
            }
        }
        else {
           if(gotLibs()[3]) {
               if(doBoxplot) {   
                   dta <- data.frame(x=transx(x[,1]),y=factor(x[,2]))
                   fit <- aov(x~y,data=dta)

                   plt1 <- ggplot(dta,aes(y,x))+geom_boxplot()+ggtitle(xname)
                   plt2 <- ggQQ(fit)  
                   pushViewport(viewport(layout = grid.layout(1, 2)))
                   print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                   print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))    
               } 
               else {
                   Dt <- data() 
                   dta <- data.frame(x=transx(Dt[,1]),y=transy(Dt[,2]),z=factor(rep(" ",nrow(Dt)))) 
                   plt1 <- ggplot(dta,aes(z,x))+geom_boxplot()+coord_flip()+xlab("")+ggtitle(xname)  
                   plt2 <-  ggplot(dta,aes(x,y))+geom_point()+
                            xlab(xname)+ylab(yname)          
                   plt3 <- ggplot(dta,aes(z,y))+geom_boxplot()+xlab("")+ggtitle(yname)                    
                   pushViewport(viewport(layout = grid.layout(2, 2)))
                   print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                   print(plt2, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))    
                   print(plt3, vp = viewport(layout.pos.row = 2, layout.pos.col = 2))    
               }
           }
        }                   
   })
           
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#transformations\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/28/2014</h6>")
      
  })
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the idea of transformations</h5>"
  })

})

